@@ -41,10 +41,10 @@ Common API implementation/base classes:
     Assembly
     Projection
 
-:copyright: Copyright 2006-2011 by the PyNN team, see AUTHORS.
+:copyright: Copyright 2006-2013 by the PyNN team, see AUTHORS.
 :license: CeCILL, see LICENSE for details.
 
-$Id: common.py 957 2011-05-03 13:44:15Z apdavison $
+$Id: common.py 1258 2013-01-31 15:01:25Z apdavison $
 """
 
 import numpy, os
@@ -233,6 +233,10 @@ class IDMixin(object):
         """Set the initial value of a state variable of the cell."""
         self.parent._set_cell_initial_value(self, variable, value)
 
+    def as_view(self):
+        """Return a PopulationView containing just this cell.""" 
+        index = self.parent.id_to_index(self) 
+        return self.parent[index:index+1] 
 
 # =============================================================================
 #   Functions for simulation set-up and control
@@ -328,9 +332,9 @@ def build_connect(projection_class, connector_class):
         otherwise. Weights should be in nA or µS.
         """
         if isinstance(source, IDMixin):
-            source = source.parent
+            source = source.as_view()
         if isinstance(target, IDMixin):
-            target = target.parent
+            target = target.as_view()
         connector = connector_class(p_connect=p, weights=weight, delays=delay)
         return projection_class(source, target, connector, target=synapse_type, rng=rng)
     return connect
@@ -637,7 +641,8 @@ class BasePopulation(object):
         logger.debug("In Population '%s', initialising %s to %s" % (self.label, variable, value))
         if isinstance(value, random.RandomDistribution):
             initial_value = value.next(n=self.all_cells.size, mask_local=self._mask_local)
-            assert len(initial_value) == self.local_size, "%d != %d" % (len(initial_value), self.local_size)
+            if self.local_size > 1:
+                assert len(initial_value) == self.local_size, "%d != %d" % (len(initial_value), self.local_size)
         else:
             initial_value = value
         self.initial_values[variable] = core.LazyArray(initial_value, shape=(self.local_size,))
