@@ -1,26 +1,31 @@
 # encoding: utf-8
 """
 nrnpython implementation of the PyNN API.
- 
+
+:copyright: Copyright 2006-2013 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
+
 $Id:__init__.py 188 2008-01-29 10:03:59Z apdavison $
 """
 __version__ = "$Rev: 191 $"
 
 from pyNN.random import *
 from pyNN.neuron import simulator
-from pyNN import common, recording, __doc__
+from pyNN import common, recording as base_recording, space, __doc__
 common.simulator = simulator
-recording.simulator = simulator
+base_recording.simulator = simulator
 
-from pyNN.neuron.cells import *
+from pyNN.neuron.standardmodels.cells import *
 from pyNN.neuron.connectors import *
-from pyNN.neuron.synapses import *
+from pyNN.neuron.standardmodels.synapses import *
 from pyNN.neuron.electrodes import *
-from pyNN.neuron.recording import *
-
+from pyNN.neuron.recording import Recorder
+from pyNN import standardmodels
 import numpy
 import logging
 
+from neuron import h
+
 logger = logging.getLogger("PyNN")
 
 # ==============================================================================
@@ -29,7 +34,7 @@ logger = logging.getLogger("PyNN")
 
 def list_standard_models():
     """Return a list of all the StandardCellType classes available for this simulator."""
-    return [obj for obj in globals().values() if isinstance(obj, type) and issubclass(obj, common.StandardCellType)]
+    return [obj.__name__ for obj in globals().values() if isinstance(obj, type) and issubclass(obj, standardmodels.StandardCellType)]
 
 # ==============================================================================
 #   Functions for simulation set-up and control
@@ -57,12 +62,19 @@ def setup(timestep=0.1, min_delay=0.1, max_delay=10.0, **extra_params):
     simulator.state.max_delay = max_delay
     if extra_params.has_key('use_cvode'):
         simulator.state.cvode.active(int(extra_params['use_cvode']))
+        if extra_params.has_key('rtol'):
+            simulator.state.cvode.rtol(float(extra_params['rtol']))
+        if extra_params.has_key('atol'):
+            simulator.state.cvode.atol(float(extra_params['atol']))
+    if extra_params.has_key('default_maxstep'):
+        simulator.state.default_maxstep=float(extra_params['default_maxstep'])
     return rank()
 
 def end(compatible_output=True):
     """Do any necessary cleaning up before exiting."""
     for recorder in simulator.recorder_list:
         recorder.write(gather=True, compatible_output=compatible_output)
+    simulator.recorder_list = []
     #simulator.finalize()
         
 def run(simtime):
@@ -72,6 +84,8 @@ def run(simtime):
     
 reset = common.reset
 
+initialize = common.initialize
+
 # ==============================================================================
 #   Functions returning information about the simulation state
 # ==============================================================================
@@ -81,23 +95,8 @@ get_time_step = common.get_time_step
 get_min_delay = common.get_min_delay
 get_max_delay = common.get_max_delay
 num_processes = common.num_processes
-rank = common.rank           
+rank = common.rank
 
-# ==============================================================================
-#   Low-level API for creating, connecting and recording from individual neurons
-# ==============================================================================
-
-create = common.create
-
-connect = common.connect
-
-set = common.set
-
-record = common.build_record('spikes', simulator)
-
-record_v = common.build_record('v', simulator)
-
-record_gsyn = common.build_record('gsyn', simulator)
 
 # ==============================================================================
 #   High-level API for creating, connecting and recording from populations of
@@ -108,67 +107,58 @@ class Population(common.Population):
     """
     An array of neurons all of the same type. `Population' is used as a generic
     term intended to include layers, columns, nuclei, etc., of cells.
-    All cells have both an address (a tuple) and an id (an integer). If p is a
-    Population object, the address and id can be inter-converted using :
-    id = p[address]
-    address = p.locate(id)
     """
+    recorder_class = Recorder
     
-    def __init__(self, dims, cellclass, cellparams=None, label=None):
-        """
-        dims should be a tuple containing the population dimensions, or a single
-          integer, for a one-dimensional population.
-          e.g., (10,10) will create a two-dimensional population of size 10x10.
-        cellclass should either be a standardized cell class (a class inheriting
-        from common.StandardCellType) or a string giving the name of the
-        simulator-specific model that makes up the population.
-        cellparams should be a dict which is passed to the neuron model
-          constructor
-        label is an optional name for the population.
-        """
-        common.Population.__init__(self, dims, cellclass, cellparams, label)
-        self.recorders = {'spikes': Recorder('spikes', population=self),
-                          'v': Recorder('v', population=self),
-                          'gsyn': Recorder('gsyn', population=self)}
-
-        # Build the arrays of cell ids
-        # Cells on the local node are represented as ID objects, other cells by integers
-        # All are stored in a single numpy array for easy lookup by address
-        # The local cells are also stored in a list, for easy iteration
-        self.all_cells, self._mask_local, self.first_id, self.last_id = simulator.create_cells(cellclass, cellparams, self.size, parent=self)
-        self.local_cells = self.all_cells[self._mask_local]
-        self.all_cells = self.all_cells.reshape(self.dim)
-        self._mask_local = self._mask_local.reshape(self.dim)
-        self.cell = self.all_cells # temporary, awaiting harmonisation
-        
+    def __init__(self, size, cellclass, cellparams=None, structure=None,
+                 label=None):
+        __doc__ = common.Population.__doc__
+        common.Population.__init__(self, size, cellclass, cellparams, structure, label)
         simulator.initializer.register(self)
-        logger.info(self.describe('Creating Population "$label" of shape $dim, '+
-                                   'containing `$celltype`s with indices between $first_id and $last_id'))
-        logger.debug(self.describe())
 
-    def rset(self, parametername, rand_distr):
+    def _create_cells(self, cellclass, cellparams, n):
+        """
+        Create cells in NEURON.
+        
+        `cellclass`  -- a PyNN standard cell or a native NEURON cell class that
+                       implements an as-yet-undescribed interface.
+        `cellparams` -- a dictionary of cell parameters.
+        `n`          -- the number of cells to create
+        """
+        # this method should never be called more than once
+        # perhaps should check for that
+        assert n > 0, 'n must be a positive integer'
+        celltype = cellclass(cellparams)
+        cell_model = celltype.model
+        cell_parameters = celltype.parameters
+        self.first_id = simulator.state.gid_counter
+        self.last_id = simulator.state.gid_counter + n - 1
+        self.all_cells = numpy.array([id for id in range(self.first_id, self.last_id+1)], simulator.ID)
+        # mask_local is used to extract those elements from arrays that apply to the cells on the current node
+        self._mask_local = self.all_cells%simulator.state.num_processes==simulator.state.mpi_rank # round-robin distribution of cells between nodes
+        for i,(id,is_local) in enumerate(zip(self.all_cells, self._mask_local)):
+            self.all_cells[i] = simulator.ID(id)
+            self.all_cells[i].parent = self
+            if is_local:
+                self.all_cells[i]._build_cell(cell_model, cell_parameters)
+        simulator.initializer.register(*self.all_cells[self._mask_local])
+        simulator.state.gid_counter += n
+
+    def _native_rset(self, parametername, rand_distr):
         """
         'Random' set. Set the value of parametername to a value taken from
         rand_distr, which should be a RandomDistribution object.
         """
-        # Note that we generate enough random numbers for all cells on all nodes
-        # but use only those relevant to this node. This ensures that the
-        # sequence of random numbers does not depend on the number of nodes,
-        # provided that the same rng with the same seed is used on each node.
-        if isinstance(rand_distr.rng, NativeRNG):
-            rng = simulator.h.Random(rand_distr.rng.seed or 0)
-            native_rand_distr = getattr(rng, rand_distr.name)
-            rarr = [native_rand_distr(*rand_distr.parameters)] + [rng.repick() for i in range(self.all_cells.size-1)]
-        else:
-            rarr = rand_distr.next(n=self.all_cells.size, mask_local=self._mask_local.flatten())
-        rarr = numpy.array(rarr)
-        logger.info("%s.rset('%s', %s)", self.label, parametername, rand_distr)
-        for cell,val in zip(self, rarr):
-            setattr(cell, parametername, val)
+        assert isinstance(rand_distr.rng, NativeRNG)
+        rng = simulator.h.Random(rand_distr.rng.seed or 0)
+        native_rand_distr = getattr(rng, rand_distr.name)
+        rarr = [native_rand_distr(*rand_distr.parameters)] + [rng.repick() for i in range(self.all_cells.size-1)]
+        self.tset(parametername, rarr)
 
 
+PopulationView = common.PopulationView
+Assembly = common.Assembly
 
-        
 class Projection(common.Projection):
     """
     A container for all the connections of a given type (same synapse type and
@@ -206,11 +196,12 @@ class Projection(common.Projection):
         
         
         ## Deal with short-term synaptic plasticity
-        if self.short_term_plasticity_mechanism:
-            U = self._short_term_plasticity_parameters['U']
-            tau_rec = self._short_term_plasticity_parameters['tau_rec']
-            tau_facil = self._short_term_plasticity_parameters['tau_facil']
-            u0 = self._short_term_plasticity_parameters['u0']
+        if self.synapse_dynamics and self.synapse_dynamics.fast:
+            # need to check it is actually the Ts-M model, even though that is the only one at present!
+            U = self.synapse_dynamics.fast.parameters['U']
+            tau_rec = self.synapse_dynamics.fast.parameters['tau_rec']
+            tau_facil = self.synapse_dynamics.fast.parameters['tau_facil']
+            u0 = self.synapse_dynamics.fast.parameters['u0']
             for cell in self.post:
                 cell._cell.set_Tsodyks_Markram_synapses(self.synapse_type, U, tau_rec, tau_facil, u0)
             synapse_model = 'Tsodyks-Markram'
@@ -227,7 +218,7 @@ class Projection(common.Projection):
         logger.info("--- Projection[%s].__init__() ---" %self.label)
                
         ## Deal with long-term synaptic plasticity
-        if self.long_term_plasticity_mechanism:
+        if self.synapse_dynamics and self.synapse_dynamics.slow:
             ddf = self.synapse_dynamics.slow.dendritic_delay_fraction
             if ddf > 0.5 and num_processes() > 1:
                 # depending on delays, can run into problems with the delay from the
@@ -236,9 +227,11 @@ class Projection(common.Projection):
                 # node with the pre-synaptic neurons for ddf>0.5 and on the node
                 # with the post-synaptic neuron (as is done now) for ddf<0.5
                 raise NotImplementedError("STDP with dendritic_delay_fraction > 0.5 is not yet supported for parallel computation.")
-            self._stdp_parameters['allow_update_on_post'] = int(False) # for compatibility with NEST
+            stdp_parameters = self.synapse_dynamics.slow.all_parameters
+            stdp_parameters['allow_update_on_post'] = int(False) # for compatibility with NEST
+            long_term_plasticity_mechanism = self.synapse_dynamics.slow.possible_models
             for c in self.connections:
-                c.useSTDP(self.long_term_plasticity_mechanism, self._stdp_parameters, ddf)
+                c.useSTDP(long_term_plasticity_mechanism, stdp_parameters, ddf)
         
         # Check none of the delays are out of bounds. This should be redundant,
         # as this should already have been done in the Connector object, so
@@ -284,6 +277,22 @@ class Projection(common.Projection):
         self.setDelays(rarr)
 
 
-Space = common.Space
+Space = space.Space
+
+# ==============================================================================
+#   Low-level API for creating, connecting and recording from individual neurons
+# ==============================================================================
+
+create = common.build_create(Population)
+
+connect = common.build_connect(Projection, FixedProbabilityConnector)
+
+set = common.set
+
+record = common.build_record('spikes', simulator)
+
+record_v = common.build_record('v', simulator)
+
+record_gsyn = common.build_record('gsyn', simulator)
 
 # ==============================================================================
