@@ -2,11 +2,17 @@
 """
 Standard cells for the neuron module.
 
-$Id: cells.py 704 2010-02-03 14:18:33Z apdavison $
+:copyright: Copyright 2006-2013 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
+
+$Id: cells.py 1258 2013-01-31 15:01:25Z apdavison $
 """
 
-from pyNN import common, cells
+from pyNN.standardmodels import cells, build_translations
+from pyNN.models import BaseCellType
+from pyNN import errors
 from neuron import h, nrn, hclass
+from simulator import state
 from math import pi
 import logging
 
@@ -30,6 +36,10 @@ def _new_property(obj_hierarchy, attr_name):
     return property(fset=set, fget=get)
 
 
+class NativeCellType(BaseCellType):
+    pass
+
+
 class SingleCompartmentNeuron(nrn.Section):
     """docstring"""
     
@@ -39,17 +49,17 @@ class SingleCompartmentNeuron(nrn.Section):
     }
 
     def __init__(self, syn_type, syn_shape, c_m, i_offset,
-                 v_init, tau_e, tau_i, e_e, e_i):
+                 tau_e, tau_i, e_e, e_i):
         
         # initialise Section object with 'pas' mechanism
         nrn.Section.__init__(self)
         self.seg = self(0.5)
         self.L = 100
         self.seg.diam = 1000/pi # gives area = 1e-3 cm2
-                
+        
+        self.source_section = self
         self.syn_type = syn_type
         self.syn_shape = syn_shape
-        self.v_init = v_init
         
         # insert synapses
         assert syn_type in ('current', 'conductance'), "syn_type must be either 'current' or 'conductance'. Actual value is %s" % syn_type
@@ -57,9 +67,13 @@ class SingleCompartmentNeuron(nrn.Section):
         synapse_model = StandardIF.synapse_models[syn_type][syn_shape]
         self.esyn = synapse_model(0.5, sec=self)
         self.isyn = synapse_model(0.5, sec=self)
-        if self.syn_type == 'conductance' and self.syn_shape == 'exp':
-            self.esyn_TM = h.tmgsyn(0.5, sec=self)
-            self.isyn_TM = h.tmgsyn(0.5, sec=self)
+        if self.syn_shape == 'exp':
+            if self.syn_type == 'conductance':
+                self.esyn_TM = h.tmgsyn(0.5, sec=self)
+                self.isyn_TM = h.tmgsyn(0.5, sec=self)
+            else:
+                self.esyn_TM = h.tmisyn(0.5, sec=self)
+                self.isyn_TM = h.tmisyn(0.5, sec=self)
         
         # insert current source
         self.stim = h.IClamp(0.5, sec=self)
@@ -67,11 +81,13 @@ class SingleCompartmentNeuron(nrn.Section):
         self.stim.dur = 1e12
         self.stim.amp = i_offset
 
-        # for recording spikes
+        # for recording
         self.spike_times = h.Vector(0)
+        self.traces = {}
         self.gsyn_trace = {}
         self.recording_time = 0
         
+        self.v_init = None
 
     @property
     def excitatory(self):
@@ -138,6 +154,8 @@ class SingleCompartmentNeuron(nrn.Section):
         if active:
             rec = h.NetCon(self.source, None)
             rec.record(self.spike_times)
+        else:
+            self.spike_times = h.Vector(0)
     
     def record_v(self, active):
         if active:
@@ -169,18 +187,14 @@ class SingleCompartmentNeuron(nrn.Section):
             if self.recording_time == 0:
                 self.record_times = None
     
-    def memb_init(self, v_init=None):
-        if v_init:
-            self.v_init = v_init
+    def memb_init(self):
         assert self.v_init is not None, "cell is a %s" % self.__class__.__name__
         for seg in self:
             seg.v = self.v_init
         #self.seg.v = self.v_init
 
     def set_Tsodyks_Markram_synapses(self, ei, U, tau_rec, tau_facil, u0):
-        if self.syn_type == 'current':
-            raise Exception("Tsodyks-Markram mechanism only available for conductance-based synapses.")
-        elif self.syn_shape == 'alpha':
+        if self.syn_shape == 'alpha':
             raise Exception("Tsodyks-Markram mechanism not available for alpha-function-shaped synapses.")
         elif ei == 'excitatory':
             syn = self.esyn_TM
@@ -201,10 +215,11 @@ class SingleCompartmentNeuron(nrn.Section):
 class LeakySingleCompartmentNeuron(SingleCompartmentNeuron):
     
     def __init__(self, syn_type, syn_shape, tau_m, c_m, v_rest, i_offset,
-                 v_init, tau_e, tau_i, e_e, e_i):
+                 tau_e, tau_i, e_e, e_i):
         SingleCompartmentNeuron.__init__(self, syn_type, syn_shape, c_m, i_offset,
-                                         v_init, tau_e, tau_i, e_e, e_i)
+                                         tau_e, tau_i, e_e, e_i)
         self.insert('pas')
+        self.v_init = v_rest # default value
         
     def __set_tau_m(self, value):
         #print "setting tau_m to", value, "cm =", self.seg.cm
@@ -235,13 +250,11 @@ class StandardIF(LeakySingleCompartmentNeuron):
     
     def __init__(self, syn_type, syn_shape, tau_m=20, c_m=1.0, v_rest=-65,
                  v_thresh=-55, t_refrac=2, i_offset=0, v_reset=None,
-                 v_init=None, tau_e=5, tau_i=5, e_e=0, e_i=-70):
-        if v_init is None:
-            v_init = v_rest
+                 tau_e=5, tau_i=5, e_e=0, e_i=-70):
         if v_reset is None:
             v_reset = v_rest
         LeakySingleCompartmentNeuron.__init__(self, syn_type, syn_shape, tau_m, c_m, v_rest,
-                                              i_offset, v_init, tau_e, tau_i, e_e, e_i)       
+                                              i_offset, tau_e, tau_i, e_e, e_i)       
         # insert spike reset mechanism
         self.spike_reset = h.ResetRefrac(0.5, sec=self)
         self.spike_reset.vspike = 40 # (mV) spike height
@@ -249,7 +262,7 @@ class StandardIF(LeakySingleCompartmentNeuron):
         
         # process arguments
         self.parameter_names = ['c_m', 'tau_m', 'v_rest', 'v_thresh', 't_refrac',   # 'c_m' must come before 'tau_m'
-                                'i_offset', 'v_reset', 'v_init', 'tau_e', 'tau_i']
+                                'i_offset', 'v_reset', 'tau_e', 'tau_i']
         if syn_type == 'conductance':
             self.parameter_names.extend(['e_e', 'e_i'])
         self.set_parameters(locals())
@@ -265,13 +278,11 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     
     def __init__(self, syn_type, syn_shape, tau_m=20, c_m=1.0, v_rest=-65,
                  v_thresh=-55, t_refrac=2, i_offset=0,
-                 v_init=None, tau_e=5, tau_i=5, e_e=0, e_i=-70,
+                 tau_e=5, tau_i=5, e_e=0, e_i=-70,
                  v_spike=0.0, v_reset=-70.6, A=4.0, B=0.0805, tau_w=144.0,
-                 w_init=0.0, delta=2.0):
-        if v_init is None:
-            v_init = v_rest
+                 delta=2.0):
         LeakySingleCompartmentNeuron.__init__(self, syn_type, syn_shape, tau_m,
-                                              c_m, v_rest, i_offset, v_init,
+                                              c_m, v_rest, i_offset,
                                               tau_e, tau_i, e_e, e_i)
     
         # insert Brette-Gerstner spike mechanism
@@ -279,11 +290,12 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
         self.source = self.seg._ref_v
         
         self.parameter_names = ['c_m', 'tau_m', 'v_rest', 'v_thresh', 't_refrac',
-                                'i_offset', 'v_reset', 'v_init', 'tau_e', 'tau_i',
-                                'A', 'B', 'tau_w', 'delta', 'w_init', 'v_spike']
+                                'i_offset', 'v_reset', 'tau_e', 'tau_i',
+                                'A', 'B', 'tau_w', 'delta', 'v_spike']
         if syn_type == 'conductance':
             self.parameter_names.extend(['e_e', 'e_i'])
         self.set_parameters(locals())
+        self.w_init = None
     
     v_thresh = _new_property('adexp', 'vthresh')
     v_reset  = _new_property('adexp', 'vreset')
@@ -293,8 +305,6 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     ## using 'A' because for some reason, cell.a gives the error "NameError: a, the mechanism does not exist at PySec_170bb70(0.5)"   
     tau_w    = _new_property('adexp',  'tauw')
     delta    = _new_property('adexp',  'delta')
-    w_init   = _new_property('adexp',  'winit') # w_init not defined in .mod file - needs to be
-    #v_init   = _new_property('seg',  'v') #?? something involving FInitialize
     
     def __set_v_spike(self, value):
         self.adexp.vspike = value
@@ -327,26 +337,81 @@ class BretteGerstnerIF(LeakySingleCompartmentNeuron):
     def get_threshold(self):
         return self.adexp.vspike
 
+    def memb_init(self):
+        assert self.v_init is not None, "cell is a %s" % self.__class__.__name__
+        assert self.w_init is not None
+        for seg in self:
+            seg.v = self.v_init
+            seg.w = self.w_init
+
+
+class GsfaGrrIF(StandardIF):
+    """docstring"""
+    
+    def __init__(self, syn_type, syn_shape, tau_m=10.0, c_m=1.0, v_rest=-70.0,
+                 v_thresh=-57.0, t_refrac=0.1, i_offset=0.0,
+                 tau_e=1.5, tau_i=10.0, e_e=0.0, e_i=-75.0,
+                 v_spike=0.0, v_reset=-70.0, q_rr=3214.0, q_sfa=14.48,
+                 e_rr=-70.0, e_sfa=-70.0,
+                 tau_rr=1.97, tau_sfa=110.0):
+
+        StandardIF.__init__(self, syn_type, syn_shape, tau_m, c_m, v_rest,
+                            v_thresh, t_refrac, i_offset, v_reset,
+                            tau_e, tau_i, e_e, e_i)
+    
+        # insert GsfaGrr mechanism
+        self.gsfa_grr = h.GsfaGrr(0.5, sec=self)
+        self.v_thresh = v_thresh
+        
+        self.parameter_names = ['c_m', 'tau_m', 'v_rest', 'v_thresh', 'v_reset',
+                                't_refrac', 'tau_e', 'tau_i', 'i_offset',
+                                'e_rr', 'e_sfa', 'q_rr', 'q_sfa', 'tau_rr', 'tau_sfa']
+        if syn_type == 'conductance':
+            self.parameter_names.extend(['e_e', 'e_i'])
+        self.set_parameters(locals())
+
+    q_sfa        = _new_property('gsfa_grr',  'q_s')
+    q_rr        = _new_property('gsfa_grr',  'q_r')
+    tau_sfa        = _new_property('gsfa_grr',  'tau_s')
+    tau_rr        = _new_property('gsfa_grr',  'tau_r')
+    e_sfa = _new_property('gsfa_grr',  'E_s')
+    e_rr = _new_property('gsfa_grr',  'E_r')
+    
+    def __set_v_thresh(self, value):
+        self.spike_reset.vthresh = value
+        # this can fail on constructor
+        try:
+            self.gsfa_grr.vthresh = value
+        except AttributeError:
+            pass
+        
+    def __get_v_thresh(self):
+        return self.spike_reset.vthresh
+    v_thresh = property(fget=__get_v_thresh, fset=__set_v_thresh)
+
+
 class SingleCompartmentTraub(SingleCompartmentNeuron):
     
     def __init__(self, syn_type, syn_shape, c_m=1.0, e_leak=-65,
-                 i_offset=0, v_init=None, tau_e=5, tau_i=5, e_e=0, e_i=-70,
-                 gbar_Na=20000, gbar_K=6000, g_leak=10, ena=50,
+                 i_offset=0, tau_e=5, tau_i=5, e_e=0, e_i=-70,
+                 gbar_Na=20e-3, gbar_K=6e-3, g_leak=0.01e-3, ena=50,
                  ek=-90, v_offset=-63):
-        if v_init is None:
-            v_init = v_rest
+        """
+        Conductances are in millisiemens (S/cm2, since A = 1e-3)
+        """
         SingleCompartmentNeuron.__init__(self, syn_type, syn_shape, c_m, i_offset,
-                                         v_init, tau_e, tau_i, e_e, e_i)
+                                         tau_e, tau_i, e_e, e_i)
         self.source = self.seg._ref_v
         self.insert('k_ion')
         self.insert('na_ion')
         self.insert('hh_traub')
-        self.parameter_names = ['c_m', 'e_leak', 'i_offset', 'v_init', 'tau_e',
+        self.parameter_names = ['c_m', 'e_leak', 'i_offset', 'tau_e',
                                 'tau_i', 'gbar_Na', 'gbar_K', 'g_leak', 'ena',
                                 'ek', 'v_offset']
         if syn_type == 'conductance':
             self.parameter_names.extend(['e_e', 'e_i'])
         self.set_parameters(locals())
+        self.v_init = e_leak # default value
 
     # not sure ena and ek are handled correctly
     
@@ -359,6 +424,12 @@ class SingleCompartmentTraub(SingleCompartmentNeuron):
     def get_threshold(self):
         return 10.0
     
+    def record(self, active):
+        if active:
+            rec = h.NetCon(self.source, None, sec=self)
+            rec.record(self.spike_times)
+            
+
     
 class RandomSpikeSource(hclass(h.NetStimFD)):
     
@@ -372,6 +443,8 @@ class RandomSpikeSource(hclass(h.NetStimFD)):
         self.spike_times = h.Vector(0)
         self.source = self
         self.switch = h.NetCon(None, self)
+        self.source_section = None
+        self.seed(state.mpi_rank) # should allow user to set specific seeds somewhere, e.g. in setup()
 
     def _set_interval(self, value):
         self.switch.weight[0] = -1
@@ -396,12 +469,13 @@ class VectorSpikeSource(hclass(h.VecStim)):
     def __init__(self, spike_times=[]):
         self.spike_times = spike_times
         self.source = self
+        self.source_section = None
             
     def _set_spike_times(self, spike_times):
         try:
             self._spike_times = h.Vector(spike_times)
         except RuntimeError:
-            raise common.InvalidParameterValueError("spike_times must be an array of floats")
+            raise errors.InvalidParameterValueError("spike_times must be an array of floats")
         self.play(self._spike_times)
             
     def _get_spike_times(self):
@@ -416,232 +490,3 @@ class VectorSpikeSource(hclass(h.VecStim)):
         we need to provide a stub for consistency with other models.
         """
         pass
-    
-            
-# == Standard cells ============================================================
-
-class IF_curr_alpha(cells.IF_curr_alpha):
-    """Leaky integrate and fire model with fixed threshold and alpha-function-
-    shaped post-synaptic current."""
-    
-    translations = common.build_translations(
-        ('tau_m',      'tau_m'),
-        ('cm',         'c_m'),
-        ('v_rest',     'v_rest'),
-        ('v_thresh',   'v_thresh'),
-        ('v_reset',    'v_reset'),
-        ('tau_refrac', 't_refrac'),
-        ('i_offset',   'i_offset'),
-        ('tau_syn_E',  'tau_e'),
-        ('tau_syn_I',  'tau_i'),
-        ('v_init',     'v_init'),
-    )
-    model = StandardIF
-    
-    def __init__(self, parameters):
-        cells.IF_curr_alpha.__init__(self, parameters) # checks supplied parameters and adds default
-                                                        # values for not-specified parameters.
-        self.parameters['syn_type']  = 'current'
-        self.parameters['syn_shape'] = 'alpha'
-
-class IF_curr_exp(cells.IF_curr_exp):
-    """Leaky integrate and fire model with fixed threshold and
-    decaying-exponential post-synaptic current. (Separate synaptic currents for
-    excitatory and inhibitory synapses."""
-    
-    translations = common.build_translations(
-        ('tau_m',      'tau_m'),
-        ('cm',         'c_m'),
-        ('v_rest',     'v_rest'),
-        ('v_thresh',   'v_thresh'),
-        ('v_reset',    'v_reset'),
-        ('tau_refrac', 't_refrac'),
-        ('i_offset',   'i_offset'),
-        ('tau_syn_E',  'tau_e'),
-        ('tau_syn_I',  'tau_i'),
-        ('v_init',     'v_init'),
-    )
-    model = StandardIF
-    
-    def __init__(self, parameters):
-        cells.IF_curr_exp.__init__(self, parameters)
-        self.parameters['syn_type']  = 'current'
-        self.parameters['syn_shape'] = 'exp'
-
-
-class IF_cond_alpha(cells.IF_cond_alpha):
-    """Leaky integrate and fire model with fixed threshold and alpha-function-
-    shaped post-synaptic conductance."""
-    
-    translations = common.build_translations(
-        ('tau_m',      'tau_m'),
-        ('cm',         'c_m'),
-        ('v_rest',     'v_rest'),
-        ('v_thresh',   'v_thresh'),
-        ('v_reset',    'v_reset'),
-        ('tau_refrac', 't_refrac'),
-        ('i_offset',   'i_offset'),
-        ('tau_syn_E',  'tau_e'),
-        ('tau_syn_I',  'tau_i'),
-        ('v_init',     'v_init'),
-        ('e_rev_E',    'e_e'),
-        ('e_rev_I',    'e_i')
-    )
-    model = StandardIF
-    
-    def __init__(self, parameters):
-        cells.IF_cond_alpha.__init__(self, parameters) # checks supplied parameters and adds default
-                                                       # values for not-specified parameters.
-        self.parameters['syn_type']  = 'conductance'
-        self.parameters['syn_shape'] = 'alpha'
-
-
-class IF_cond_exp(cells.IF_cond_exp):
-    """Leaky integrate and fire model with fixed threshold and 
-    exponentially-decaying post-synaptic conductance."""
-    
-    translations = common.build_translations(
-        ('tau_m',      'tau_m'),
-        ('cm',         'c_m'),
-        ('v_rest',     'v_rest'),
-        ('v_thresh',   'v_thresh'),
-        ('v_reset',    'v_reset'),
-        ('tau_refrac', 't_refrac'),
-        ('i_offset',   'i_offset'),
-        ('tau_syn_E',  'tau_e'),
-        ('tau_syn_I',  'tau_i'),
-        ('v_init',     'v_init'),
-        ('e_rev_E',    'e_e'),
-        ('e_rev_I',    'e_i')
-    )
-    model = StandardIF
-    
-    def __init__(self, parameters):
-        cells.IF_cond_exp.__init__(self, parameters) # checks supplied parameters and adds default
-                                                       # values for not-specified parameters.
-        self.parameters['syn_type']  = 'conductance'
-        self.parameters['syn_shape'] = 'exp'
-
-
-class IF_facets_hardware1(cells.IF_facets_hardware1):
-    """Leaky integrate and fire model with conductance-based synapses and fixed
-    threshold as it is resembled by the FACETS Hardware Stage 1. For further
-    details regarding the hardware model see the FACETS-internal Wiki:
-    https://facets.kip.uni-heidelberg.de/private/wiki/index.php/WP7_NNM
-    """
-
-    translations = common.build_translations(
-        ('v_rest',     'v_rest'),
-        ('v_thresh',   'v_thresh'),
-        ('v_reset',    'v_reset'),
-        ('g_leak',     'tau_m',    "0.2*1000.0/g_leak", "0.2*1000.0/tau_m"),
-        ('tau_syn_E',  'tau_e'),
-        ('tau_syn_I',  'tau_i'),
-        ('e_rev_I',    'e_i')
-    )
-    model = StandardIF
-
-    def __init__(self, parameters):
-        cells.IF_facets_hardware1.__init__(self, parameters)
-        self.parameters['syn_type']  = 'conductance'
-        self.parameters['syn_shape'] = 'exp'
-        self.parameters['i_offset']  = 0.0
-        self.parameters['c_m']       = 0.2
-        self.parameters['t_refrac']  = 1.0
-        self.parameters['e_e']       = 0.0
-    
-       
-class HH_cond_exp(cells.HH_cond_exp):
-    
-    translations = common.build_translations(
-        ('gbar_Na',    'gbar_Na', 1e-6),   
-        ('gbar_K',     'gbar_K', 1e-6),    
-        ('g_leak',     'g_leak', 1e-6),    
-        ('cm',         'c_m'),  
-        ('v_offset',   'v_offset'),
-        ('e_rev_Na',   'ena'),
-        ('e_rev_K',    'ek'), 
-        ('e_rev_leak', 'e_leak'),
-        ('e_rev_E',    'e_e'),
-        ('e_rev_I',    'e_i'),
-        ('tau_syn_E',  'tau_e'),
-        ('tau_syn_I',  'tau_i'),
-        ('i_offset',   'i_offset'),
-        ('v_init',     'v_init'),
-    )
-    model = SingleCompartmentTraub
-
-    def __init__(self, parameters):
-        cells.HH_cond_exp.__init__(self, parameters) # checks supplied parameters and adds default
-                                                     # values for not-specified parameters.
-        self.parameters['syn_type']  = 'conductance'
-        self.parameters['syn_shape'] = 'exp'
-
-class SpikeSourcePoisson(cells.SpikeSourcePoisson):
-    """Spike source, generating spikes according to a Poisson process."""
-
-    translations = common.build_translations(
-        ('start',    'start'),
-        ('rate',     '_interval',  "1000.0/rate",  "1000.0/_interval"),
-        ('duration', 'duration'),
-    )
-    model = RandomSpikeSource
-
-
-class SpikeSourceArray(cells.SpikeSourceArray):
-    """Spike source generating spikes at the times given in the spike_times array."""
-
-    translations = common.build_translations(
-        ('spike_times', 'spike_times'),
-    )
-    model = VectorSpikeSource
-       
-        
-class EIF_cond_alpha_isfa_ista(cells.EIF_cond_alpha_isfa_ista):
-    """
-    Exponential integrate and fire neuron with spike triggered and sub-threshold
-    adaptation currents (isfa, ista reps.) according to:
-    
-    Brette R and Gerstner W (2005) Adaptive Exponential Integrate-and-Fire Model as
-    an Effective Description of Neuronal Activity. J Neurophysiol 94:3637-3642
-
-    See also: IF_cond_exp_gsfa_grr
-    """
-    
-    translations = common.build_translations(
-        ('v_init',     'v_init'),
-        ('w_init',     'w_init'),
-        ('cm',         'c_m'),
-        ('tau_refrac', 't_refrac'), 
-        ('v_spike',    'v_spike'),
-        ('v_reset',    'v_reset'),
-        ('v_rest',     'v_rest'),
-        ('tau_m',      'tau_m'),
-        ('i_offset',   'i_offset'), 
-        ('a',          'A',        0.001), # nS --> uS
-        ('b',          'B'),
-        ('delta_T',    'delta'), 
-        ('tau_w',      'tau_w'), 
-        ('v_thresh',   'v_thresh'), 
-        ('e_rev_E',    'e_e'),
-        ('tau_syn_E',  'tau_e'), 
-        ('e_rev_I',    'e_i'), 
-        ('tau_syn_I',  'tau_i'),
-    )
-    model = BretteGerstnerIF
-    
-    def __init__(self, parameters):
-        cells.EIF_cond_alpha_isfa_ista.__init__(self, parameters)
-        self.parameters['syn_type']  = 'conductance'
-        self.parameters['syn_shape'] = 'alpha'
-
-class EIF_cond_exp_isfa_ista(cells.EIF_cond_exp_isfa_ista):
-    """Like EIF_cond_alpha_isfa_ista, but with single-exponential synapses."""
-    
-    translations = EIF_cond_alpha_isfa_ista.translations
-    model = BretteGerstnerIF
-    
-    def __init__(self, parameters):
-        cells.EIF_cond_exp_isfa_ista.__init__(self, parameters)
-        self.parameters['syn_type']  = 'conductance'
-        self.parameters['syn_shape'] = 'exp'
\ No newline at end of file
