@@ -1,7 +1,10 @@
 """
 Connection method classes for pcsim
 
-$Id: connectors.py 626 2009-06-01 15:40:56Z apdavison $
+:copyright: Copyright 2006-2013 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
+
+$Id: connectors.py 1258 2013-01-31 15:01:25Z apdavison $
 """
 
 from pyNN.connectors import AllToAllConnector, \
@@ -11,7 +14,8 @@ from pyNN.connectors import AllToAllConnector, \
                             FromListConnector, \
                             FromFileConnector, \
                             FixedNumberPreConnector, \
-                            FixedNumberPostConnector
+                            FixedNumberPostConnector, \
+                            SmallWorldConnector
 
 
 
