@@ -5,7 +5,10 @@ Classes:
     DCSource           -- a single pulse of current of constant amplitude.
     StepCurrentSource  -- a step-wise time-varying current.
 
-$Id: electrodes.py 708 2010-02-12 15:02:09Z apdavison $
+:copyright: Copyright 2006-2013 by the PyNN team, see AUTHORS.
+:license: CeCILL, see LICENSE for details.
+
+$Id: electrodes.py 1258 2013-01-31 15:01:25Z apdavison $
 """
 
 import numpy
@@ -23,7 +26,7 @@ class CurrentSource(object):
         else:
             delay = simulator.state.min_delay # perhaps it would be better to replicate the current source on each node, to avoid this delay
         for cell in cell_list:
-            if cell.local and 'v' not in cell.cellclass.recordable:
+            if cell.local and not cell.celltype.injectable:
                 raise TypeError("Can't inject current into a spike source.")
             c = simulator.net.connect(self.input_node, cell, pypcsim.StaticAnalogSynapse(delay=0.001*delay))
             self.connections.append(c)
@@ -80,4 +83,4 @@ class DCSource(StepCurrentSource):
         times = [0.0, start, (stop or 1e99)]
         amplitudes = [0.0, amplitude, 0.0]
         StepCurrentSource.__init__(self, times, amplitudes)
-        
\ No newline at end of file
+        
